% KDEDEMO2    Demonstrate UIControls with 2D Kernel Density Estimates.
%
%             Example: data=randn(500,2); (optional - defines data)
%                      kdedemo2    
%
%             Menus: (a) DATA - A choice of three 2D data sets.
%                    (b) COLOUR - A choice of several colormaps.
%
%             Pop-ups: (a) KERNEL - A choice of four well-known 2D kernel functions.
%                      (b) PLOT - Choice of Surface, Contour or Percent Contour plots.
%                      (c) HSELECT - A choice of several data-based h selection strategies.
%
%             Check-boxes: (a) ADAPTIVE - Turns adaptive method on/off,
%                          (b) SHADNG - Turns interpolated shading on/off.
%
%             Push buttons: (a) END - Quit demonstration,
%                           (b) HELP - Output these comments.
%
%             Sliders: (a) LR - Sets azimuth angle of view,
%                      (b) UD - Sets elevation angle of view.
%
%             Editable text: (a) H - Click here to enter a value for H.  Values 
%                                of H=0 cause the routine to use an 'optimal' 
%                                value.  If entering >1 value use vector notation, 
%                                eg [2,1] or [2,0].
%                            (b) CONTOUR PERCENTAGES - eg [25,50,75] or [50,100] or
%                                [10:10:90]. 
% 
%             Christian C. Beardah 1995  

echo off

load kddata;

if exist('data')==0,
  data=lukesh1(:,[2,5]);
  import=0;
  else,
    data=data(:,1:2);
    import=1;
end;

% Initialise

kernel='knorm';

if exist('nmesh')==0,
  nmesh=32;
end;

azim=-37.5;
elev=30;
adapt=0;
h=[hns(data(:,1),kernel) hns(data(:,2),kernel)];
pc=[25,50,75];
sym='-';

figure('Name','Bi-variate KDE demo');

gcfpos = zeros(1,4);

% x,y are vectors, z a matrix

[x,y,z]=kdfft2(data,kernel,nmesh,h);

xlabel('x');
ylabel('y');
zlabel('Density');

view(azim,elev);

% Callback strings

ends = 'clear global; clear; close;';

smooths = 'if get(smoothh,''value'')==1, shading interp; else, shading faceted; end; ';

calcs = 'if get(ploth,''value'')>=3, ddata=kdint2(data,x,y,z); w=pclvl2(pc,ddata); end;';

holds = 'if get(holdh,''value'')==1, hold on; else, hold off; end;';

dview2s = 'view(2); set(udc,''value'',90); set(lrc,''value'',0); azim=0; elev=90;';

dview3s = 'view(3); set(udc,''value'',30); set(lrc,''value'',-37.5); azim=-37.5; elev=30;';

scatters = ['v=get(ploth,''value''); if v==3, if get(scatterh,''value'')==1, hold on; plot(data(:,1),data(:,2),''cx'');',... 
            'if exist(''outl''), for i=1:length(outl), text(outl(i,2),outl(i,3),int2str(outl(i,1))); end; end; hold off; else,',...
            'if get(holdh,''value'')==0, for i=1:length(pc), contour3(x,y,z,w(i,:),sym); if i==1, hold on; end; end;',...
            'hold off; elev=90; set(udc,''value'',elev); view(azim,elev);',...
            'set(scatterh,''value'',0); else, set(scatterh,''value'',1); end; end; else, set(scatterh,''value'',0); end;']; 

conts = [holds,'for i=1:length(pc), contour3(x,y,z,w(i,:),sym); if i==1, hold on; end; end; hold off;',...
         'if get(scatterh,''value'')==1, hold on, plot(data(:,1),data(:,2),''cx''); if exist(''outl''), for i=1:length(outl),',...
         'text(outl(i,2),outl(i,3),int2str(outl(i,1))); end; end; hold off; end;',...
         'elev=90; set(udc,''value'',elev); view(azim,elev);']; 

plots = ['v=get(ploth,''value''); if v==1, set(scatterh,''value'',0); surf(x,y,z);',...
          smooths,'elseif v==2, set(scatterh,''value'',0); contour3(x,y,z,8,''w''); elev=90; set(udc,''value'',elev);',...
         'elseif v==3, if ~exist(''w''),',calcs,' end;',conts,...
         'elseif v==4, if ~exist(''ddata''),',calcs,' end; [outl,index]=outl2(max(pc),data,x,y,z,ddata); outl, plot(outl(:,5),outl(:,4),''wx'');',...
         'for i=1:length(outl), text(outl(i,5),outl(i,4),int2str(outl(i,1))); end; elev=90; set(udc,''value'',elev); view(azim,elev); end;',...
         'view(azim,elev); if v<4, xlabel(''x''); ylabel(''y''); zlabel(''Rel. Freq.''); else,',...
         'xlabel(''Minimum distance from outer contour''); ylabel(''Density''); end;',...
         'if length(h)==3, title([''h = '',num2str(h(1)),'' , '',num2str(h(2)),'' , '',num2str(h(3))]);',...
         'elseif length(h)==2, title([''h = '',num2str(h(1)),'' , '',num2str(h(2))]);',...
         'else, title([''h = '',num2str(h)]); end;'];

hselects = ['clear w outl; v=get(hselecth,''value''); if v==1, h=[hns(data(:,1),kernel) hns(data(:,2),kernel)];',...
           'elseif v==2 & kernel==''knorm'', h=[hdpi(data(:,1),kernel,1) hdpi(data(:,2),kernel,1)];',...
           'elseif v==3 & kernel==''knorm'', h=[hdpi(data(:,1),kernel,2) hdpi(data(:,2),kernel,2)];',...
           'elseif v==4 & kernel==''knorm'', h=[hdpi(data(:,1),kernel,3) hdpi(data(:,2),kernel,3)];',...
           'elseif v==5 & kernel==''knorm'', h=[hste(data(:,1),kernel) hste(data(:,2),kernel)];',...
           'elseif v==6 & kernel==''knorm'', h=hbcv2(data,kernel);',...
           'else, h=[hns(data(:,1),kernel) hns(data(:,2),kernel)];',...  
           'set(hselecth,''value'',1); end; set(hvalh,''string'',[''['',num2str(h(1)),'','',num2str(h(2)),'']'']);',...
           'if adapt==0, [x,y,z]=kdfft2(data,kernel,nmesh,h,0);',...
           'else, [x,y,z,h]=kdplot(data,kernel,nmesh,adapt,h,0); end;',calcs,plots];

hvals = ['h=str2num(get(hvalh,''string'')); if min(h)<0, disp(''ERROR - negative h'');',...
         'else, if adapt==0, [x,y,z]=kdfft2(data,kernel,nmesh,h,0); clear w outl;',plots,...
         'else, [x,y,z,h]=kdplot(data,kernel,nmesh,adapt,h,0); clear w outl;',plots,'end; end;'];

adapts = ['if get(adapth,''value'')==1, adapt=1; else, adapt=0; end; clear w outl;',hvals];

pcvals = ['pc=str2num(get(pcvalh,''string'')); if min(pc)<0, disp(''ERROR - negative contour level'');',...
         'else, clear outl; w=pclvl2(pc,ddata); end; set(ploth,''value'',3);',plots];

meshs = ['nmesh=str2num(get(meshh,''string'')); if nmesh<8, disp(''ERROR - mesh too small'');',...
         'else, if adapt==0, [x,y,z]=kdfft2(data,kernel,nmesh,h,0);',...
         'else, [x,y,z,h]=kdplot(data,kernel,nmesh,adapt,h,0); end; clear w outl;',plots,'end;'];

kerpops = ['clear w outl; v=get(kerpoph,''value''); if v==1, kernel=''knorm'';',...
           'elseif v==2, kernel=''kepan'';',...
           'elseif v==3, kernel=''kbiwe'';',...
           'elseif v==4, kernel=''ktriw'';',...
           'elseif v==5, kernel=''ktria'';',...
           'elseif v==6, kernel=''krect''; end;', hselects];

% Frames

frame1h = uicontrol('style','frame','units','normal','pos',[.84 .73 .15 .2]);

frame2h = uicontrol('style','frame','units','normal','pos',[.02 .02 .175 .1]);

% Check boxes

adapth = uicontrol('style','checkbox','units','normal','pos',[.85 .86 .125 .05],...
  'string','Adaptive','call',adapts);
smoothh = uicontrol('style','checkbox','units','normal','pos',[.02 .87 .125 .05],...
  'string','Shading','call',smooths);
scatterh = uicontrol('style','checkbox','units','normal','pos',[.02 .14 .125 .05],...
  'string','Scatter','call',scatters);
holdh = uicontrol('style','checkbox','units','normal','pos',[.02 .21 .09 .05],...
  'string','Hold','call',holds);

% Pop-up menus

kerpoph = uicontrol('style','popupmenu','units','normal','pos',[.8 .94 .185 .08],...
  'string','Normal|Epanechnikov|Biweight_P|Triweight_P|Triangular_P|Rectangular_P','call',kerpops);   
ploth = uicontrol('style','popupmenu','units','normal','pos',[.015 .94 .24 .08],...
  'string','Surface|Contour|Percent Contour|Outliers','call',plots);   
hselecth = uicontrol('style','popupmenu','units','normal','pos',[.8 .67 .19 .08],...
  'string','Normal Scale|DPI 1|DPI 2|DPI 3|STE2','call',hselects);   

% Menus

datamenuh = uimenu(gcf,'label','Data');

glassh = uimenu(datamenuh,'label','Glass Chemistry (361)','callback',...
  ['clear w outl; set(scatterh,''value'',0); data=[glass(:,5) glass(:,8)];',hselects,...
   'set(hvalh,''string'',[''['',num2str(h(1)),'','',num2str(h(2)),'']'']);',...
    plots,'set(glassh,''checked'',''on''),',...
   'set(cuph,''checked'',''off''),',... 
   'set(n01h,''checked'',''off''),',... 
   'set(lmh,''checked'',''off''),',... 
   'set(maskh,''checked'',''off'')']);
maskh = uimenu(datamenuh,'label','Artifact Locations (276)','callback',...
  ['clear w outl; set(scatterh,''value'',0); data=mask4;',hselects,...
   'set(hvalh,''string'',[''['',num2str(h(1)),'','',num2str(h(2)),'']'']);',...
    plots,'set(glassh,''checked'',''off''),',...
   'set(cuph,''checked'',''off''),',... 
   'set(n01h,''checked'',''off''),',... 
   'set(lmh,''checked'',''off''),',... 
   'set(maskh,''checked'',''on'')']);
cuph = uimenu(datamenuh,'label','Cup Diam./Height (60)','callback',...
  ['clear w outl; set(scatterh,''value'',0); data=lukesh1(:,[2,5]);',hselects,...
   'set(hvalh,''string'',[''['',num2str(h(1)),'','',num2str(h(2)),'']'']);',...
    plots,'set(glassh,''checked'',''off''),',...
   'set(cuph,''checked'',''on''),',... 
   'set(n01h,''checked'',''off''),',... 
   'set(lmh,''checked'',''off''),',... 
   'set(maskh,''checked'',''off'')']);
lmh = uimenu(datamenuh,'label','Leic/Manc (105)','callback',...
  ['clear w outl; set(scatterh,''value'',0); data=lmpc(:,1:2);',hselects,...
   'set(hvalh,''string'',[''['',num2str(h(1)),'','',num2str(h(2)),'']'']);',...
    plots,'set(glassh,''checked'',''off''),',...
   'set(cuph,''checked'',''off''),',... 
   'set(n01h,''checked'',''off''),',... 
   'set(lmh,''checked'',''on''),',... 
   'set(maskh,''checked'',''off'')']);
n01h = uimenu(datamenuh,'label','N(0,1) (500)','callback',...
  ['clear w outl; set(scatterh,''value'',0); data=randn(500,2);',hselects,...
   'set(hvalh,''string'',[''['',num2str(h(1)),'','',num2str(h(2)),'']'']);',...
    plots,'set(glassh,''checked'',''off''),',...
   'set(cuph,''checked'',''off''),',... 
   'set(n01h,''checked'',''on''),',... 
   'set(lmh,''checked'',''off''),',... 
   'set(maskh,''checked'',''off'')']);

symmenuh = uimenu(gcf,'label','Print style');

solidh = uimenu(symmenuh,'label','Solid','callback',...
  ['sym=''-''; set(solidh,''checked'',''on''),',...
   'set(dashh,''checked'',''off''),',...
   'set(dotdashh,''checked'',''off''),',...
   'set(doth,''checked'',''off'')']);
dashh = uimenu(symmenuh,'label','Dashed','callback',...
  ['sym=''--''; set(solidh,''checked'',''off''),',...
   'set(dashh,''checked'',''on''),',...
   'set(dotdashh,''checked'',''off''),',...
   'set(doth,''checked'',''off'')']);
dotdashh = uimenu(symmenuh,'label','Dash-dotted','callback',...
  ['sym=''-.''; set(solidh,''checked'',''off''),',...
   'set(dashh,''checked'',''off''),',...
   'set(dotdashh,''checked'',''on''),',...
   'set(doth,''checked'',''off'')']);
doth = uimenu(symmenuh,'label','Dotted','callback',...
  ['sym='':''; set(solidh,''checked'',''off''),',...
   'set(dashh,''checked'',''off''),',...
   'set(dotdashh,''checked'',''off''),',...
   'set(doth,''checked'',''on'')']);

set(solidh,'checked','on');

if import==0,
  set(cuph,'checked','on');
end;

% Sliders

lrc = uicontrol('style','slider','units','normal','pos',[.35 .02 .25 .04],...
     'min',-90,'max',90,'value',azim,...
     'call',['azim = ceil(get(lrc,''value''));',... 
     'set(lrc,''value'',azim);',plots;]);
lrmin = uicontrol('style','text','units','normal','pos',[.305 .02 .04 .04],...
     'string',num2str(get(lrc,'min')));
lrmax = uicontrol('style','text','units','normal','pos',[.605 .02 .04 .04],...
     'string',num2str(get(lrc,'max')));

udc = uicontrol('style','slider','units','normal','pos',[.945 .36 .04 .25],...
     'min',0,'max',90,'value',elev,...
     'call',['elev = ceil(get(udc,''value''));',... 
     'set(udc,''value'',elev);',plots;]);
udmin = uicontrol('style','text','units','normal','pos',[.945 .32 .04 .035],...
     'string',num2str(get(udc,'min')));
udmax = uicontrol('style','text','units','normal','pos',[.945 .615 .04 .035],...
     'string',num2str(get(udc,'max')));


% Push buttons

endh = uicontrol('style','push','units','normal','pos',[.91 .02 .08 .06],...
        'string','End','call',ends);
dview2h = uicontrol('style','push','units','normal','pos',[.81 .02 .08 .06],...
        'string','View2','call',dview2s);
dview3h = uicontrol('style','push','units','normal','pos',[.91 .1 .08 .06],...
        'string','View3','call',dview3s);

% Editable text

hvalh = uicontrol('style','edit','units','normal','pos',[.885 .8 .09 .04],...
        'string','0.00','min',0,'call',hvals);
hvalt = uicontrol('style','text','units','normal','pos',[.86 .8 .02 .04],...
        'string','h:');

set(hvalh,'string',['[',num2str(h(1)),',',num2str(h(2)),']']);

pcvalh = uicontrol('style','edit','units','normal','pos',[.035 .03 .15 .04],...
        'string','[25,50,75]','min',0,'call',pcvals);
pcvalt = uicontrol('style','text','units','normal','pos',[.035 .07 .15 .04],...
        'string','Contour %''s:');

meshh = uicontrol('style','edit','units','normal','pos',[.92 .74 .05 .04],...
        'string',num2str(nmesh),'min',8,'call',meshs);
mesht = uicontrol('style','text','units','normal','pos',[.86 .74 .06 .04],...
        'string','mesh:');

colormenu;